import { Component, ViewChild } from "@angular/core";
import {
  FormArray,
  FormBuilder,
  FormGroup,
  NgForm,
  Validators,
} from "@angular/forms";
import { ActivatedRoute, Router } from "@angular/router";
import { BlockUI, NgBlockUI } from "ng-block-ui";
import { BuildingService } from "../../service/monitoring-api.service";
import { LoginService } from "../../service/login.service";
import { ToastrService } from "ngx-toastr";
import { NgbModal } from "@ng-bootstrap/ng-bootstrap";
import { AddEditMasterBuildingComponent } from "../../master/master-building/add-edit-master-building/add-edit-master-building.component";
import { AddEditMasterComponent } from "../../master/add-edit-master/add-edit-master.component";
import { AddEditMasterRoomComponent } from "../../master/master-room/add-edit-master-room/add-edit-master-room.component";
import Swal from "sweetalert2";

@Component({
  selector: "app-add-edit-list",
  templateUrl: "./add-edit-list.component.html",
  styleUrls: ["./add-edit-list.component.css"],
})
export class AddEditListComponent {
  // @ViewChild("f", { read: true }) userProfileForm: NgForm;

  model: any = {};
  mode: string;
  public breadcrumb: any;
  projectInfo: FormGroup;
  submitted = false;

  dataBuildingList: any;
  dataRoomList: any;
  dataRoomListSementara: any;
  dataFloorList: any;
  dataMasterStatus: any;
  roombuildingId: any;
  dataRoomBuilding: any;
  disableButton: boolean = true;
  disableButtonSave: boolean = true;
  lewatModal: boolean = false;
  // userProfileForm: FormGroup;
  public userList: FormArray;

  @BlockUI("projectInfo") blockUIProjectInfo: NgBlockUI;
  get userFormGroup() {
    return this.projectInfo.get("userArray") as FormArray;
  }

  constructor(
    private formBuilder: FormBuilder,
    private router: Router,
    private route: ActivatedRoute,
    private monitoringApiService: BuildingService,
    private authService: LoginService,
    private toastr: ToastrService,
    private modalService: NgbModal
  ) {}

  ngOnInit() {
    this.authService.startTokenCheck();
    this.authService.startTrackingActivity();
    this.route.data.subscribe((data) => {
      this.mode = data.mode;
    });
    this.route.params.subscribe((params) => {
      const id = params["id"];
      this.roombuildingId = id;
      if (id) {
        this.loadRoomBuilding(id);
      }
    });
    this.setBreadcrumb();
    

    this.projectInfo = this.formBuilder.group({
      buildingId: ["", Validators.required],
      roomId: [""],
      statusId: ["", Validators.required],
      floorId: ["", Validators.required],
      userArray: this.formBuilder.array([this.createRoom()]),
    });
    this.userList = this.projectInfo.get("userArray") as FormArray;

    this.dataListBuilding();
    this.dataListRoom();
    this.dataListMaster();
    this.dataListFloor();
  }

  createRoom(): FormGroup {
    return this.formBuilder.group({
      roomId: ["", Validators.required],
    });
  }

  addPhone() {
    this.userList.push(this.createRoom());
    this.disableButton = true;
    this.disableButtonSave = true;
  }

  removePhone(index) {
    this.userList.removeAt(index);
    this.isRoomDuplicate(index);
    if (this.userList.value.length === 0) {
      this.disableButtonSave = true;
    }
  }

  validateDouble(event: any, index: number) {
    const selectedRoomId = (event.target as HTMLSelectElement).value;
    if (this.isRoomDuplicate(index)) {
      this.disableButton = true;
      this.disableButtonSave = true;
      this.toastr.error("Room", "Data yang ada masukan double!", {
        timeOut: 2000,
        closeButton: true,
      });
    } else {
      this.disableButton = false;
      this.disableButtonSave = false;
    }
    
  }

  isRoomDuplicate(index: number): boolean {
    const selectedRooms = this.userFormGroup.value.map((room) => room.roomId);
    const currentRoomId = selectedRooms[index];
    const roomOccurrences = selectedRooms.filter(
      (roomId) => roomId === currentRoomId
    ).length;
    if (roomOccurrences === 0) {
      this.disableButton = false;
      this.disableButtonSave = false;
    }
    return roomOccurrences > 1;
  }

  setBreadcrumb() {
    if (this.isAddMode()) {
      this.breadcrumb = {
        mainlabel: "List Monitoring",
        links: [
          {
            name: "Home",
            isLink: false,
          },
          {
            name: "List Monitoring",
            isLink: false,
          },
          {
            name: "Add New List Monitoring",
            isLink: false,
          },
        ],
      };
    } else if (this.isEditMode()) {
      this.breadcrumb = {
        mainlabel: "List Monitoring",
        links: [
          {
            name: "Home",
            isLink: false,
          },
          {
            name: "List Monitoring",
            isLink: false,
          },
          {
            name: "Edit New List Monitoring",
            isLink: false,
          },
        ],
      };
    }
  }

  loadRoomBuilding(deviceId: string) {
    this.monitoringApiService
      .getRoomBuildingById(deviceId)
      .subscribe((data) => {
        this.dataRoomBuilding = data;
        this.formGetDevice(data);
      });
  }

  formGetDevice(data) {
    this.projectInfo.patchValue({
      buildingId: data.data.buildingId,
      roomId: data.data.roomId,
      floorId: data.data.floorId,
      statusId: data.data.statusId,
    });

    if (this.mode === "view") {
      this.formDisable();
    }
  }

  formDisable() {
    this.projectInfo.get("buildingId").disable();
    this.projectInfo.get("roomId").disable();
    this.projectInfo.get("statusId").disable();
  }

  isEditMode() {
    return this.mode === "edit";
  }

  isViewMode() {
    return this.mode === "view";
  }

  isAddMode() {
    return this.mode === "add";
  }

  get f() {
    return this.projectInfo.controls;
  }

  onProjectInfoSubmit() {
    this.submitted = true;
    if (this.projectInfo.invalid) {
      return;
    }
  }

  keyPress(event: any) {
    const pattern = /[0-9\+\-\ ]/;
    const inputChar = String.fromCharCode(event.charCode);
    if (event.keyCode !== 8 && !pattern.test(inputChar)) {
      event.preventDefault();
    }
  }

  dataListBuilding() {
    this.monitoringApiService.getBuildingList().subscribe((data) => {
      this.dataBuildingList = data.data
        .filter(
          (item) =>
            item.statusName.toLowerCase() === "aktif" || item.status_id === 71
        )
        .sort((a, b) => b.id - a.id);

      if (this.lewatModal) {
        this.projectInfo.patchValue({
          buildingId: this.dataBuildingList[0],
        });
        this.lewatModal = false;
      }
    });
  }

  dataListRoom() {
    this.monitoringApiService.getListRoomDataUnmap().subscribe((data) => {
      const newArray = data.data.map((item) => ({
        id: item.id,
        name: `${item.name} (${item.code})`,
      })).sort((a, b) => b.id - a.id);
      this.dataRoomList = newArray;
    });
  }
  
  trackByFn(index: number, option: any): any {
    return option.id;
  }

  dataListFloor() {
    this.monitoringApiService.getListFloorDataUnmap(6).subscribe((data) => {
      const newArray = data.data.map((item) => ({
        id: item.id,
        name: `${item.name} (${item.code})`,
        status: item.status_name
      })).sort((a, b) => b.id - a.id);
      this.dataFloorList = newArray.filter((item) => item.status.toLowerCase() === "aktif");
      if (this.lewatModal) {
        this.projectInfo.patchValue({
          floorId: this.dataFloorList[0],
        });
        this.lewatModal = false;
      }
    });
  }

  dataListMaster() {
    this.monitoringApiService.getMasterListData().subscribe((data) => {
      const dataStatus = data.data.find(
        (item) => item.name === "master_status"
      ).headerDetailParam;
      this.dataMasterStatus = dataStatus.filter(
        (item) =>
          item.statusName.toLowerCase() === "aktif" ||
          item.status.toLowerCase() === "71"
      );
    });
  }

  save() {
    if (this.mode === "add") {
      if (this.projectInfo.invalid) {
        this.markFormGroupTouched(this.projectInfo)
        return;
      }
      const formData = this.projectInfo.value;
      const transformedData = {
        buildingId: formData.buildingId,
        roomId: formData.userArray.map((room) => room.roomId),
        statusId: formData.statusId,
        floorId: formData.floorId,
      };
      this.monitoringApiService
        .postBatchBuilding(transformedData)
        .subscribe((res) => {
          this.router.navigate(["/list-monitoring"]);
        });
    } else {
      delete this.projectInfo.value.userArray;
      this.monitoringApiService
        .putBuildingRoom(this.projectInfo.value, this.roombuildingId)
        .subscribe((data) => {
          this.router.navigate(["/list-monitoring"]);
        });
    }
  }

  markFormGroupTouched(formGroup: FormGroup) {
    (Object as any).values(formGroup.controls).forEach((control) => {
      control.markAsTouched();
      if (control.controls) {
        this.markFormGroupTouched(control);
      }
    });
  }

  cancel() {
    if (this.projectInfo.dirty) { // Check if form has unsaved changes
      Swal.fire({
        title: 'Are you sure?',
        text: "You have unsaved changes. Do you really want to leave?",
        icon: 'warning',
        showCancelButton: true,
        confirmButtonColor: '#37a647',
        cancelButtonColor: '#d33',
        confirmButtonText: 'Yes, leave',
        cancelButtonText: 'No, stay'
      }).then((result) => {
        if (result.isConfirmed) {
          this.router.navigate(["/list-monitoring"]);
        }
      });
    } else {
      this.router.navigate(["/list-monitoring"]);
    }
  }

  modalAddBuilding() {
    const modalRef = this.modalService.open(AddEditMasterBuildingComponent, {
      size: "lg",
      backdrop: "static",
      keyboard: false,
    });

    modalRef.componentInstance.mode = "add";
    modalRef.result.then(
      (result) => {
        if (result) {
          if (
            this.dataBuildingList.some(
              (value) =>
                value.name.trim().toLowerCase() ===
                result.name.trim().toLowerCase()
            )
          ) {
            this.toastr.error("Warning", "Data yang anda masukan double.", {
              timeOut: 5000,
              closeButton: true,
            });
          } else {
            this.monitoringApiService
              .postMasterBuildingParam(result)
              .subscribe((res) => {
                this.dataListBuilding();
                this.lewatModal = true;
              });
          }
        }
      },
      (reason) => {
        console.log(`Dismissed: ${reason}`);
      }
    );
  }

  modalAddFloor() {
    const modalRef = this.modalService.open(AddEditMasterComponent, {
      size: "lg",
      backdrop: "static", 
      keyboard: false, 
    });

    modalRef.componentInstance.headerId = 6;
    modalRef.componentInstance.mode = "add";
    modalRef.componentInstance.floor = true;
    modalRef.result.then(
      (result) => {
        if (result) {
          this.monitoringApiService
            .postHeaderDetailParam(result)
            .subscribe((res) => {
              this.dataListFloor();
              this.lewatModal = true;
              if (res.status === 400) {
                this.toastr.error("Warning", "Data yang anda masukan double.", {
                  timeOut: 5000,
                  closeButton: true,
                });
              }
            });
        }
      },
      (reason) => {
        console.log(`Dismissed: ${reason}`);
      }
    );
  }

  modalAddNewRoom(){
    const modalRef = this.modalService.open(AddEditMasterRoomComponent, {
      size: "lg",
      backdrop: "static",
      keyboard: false,
    });

    modalRef.componentInstance.mode = "add";
    modalRef.result.then(
      (result) => {
        if (result) {
          const filteredData = {
            name: result.name,
            code: result.code.trim(),
            description: result.description,
          };
          this.monitoringApiService
            .postMasterRoomParam(filteredData)
            .subscribe((res) => {
              this.dataListRoom();
            }, (error) => {
              this.toastr.error("Error", error.error.message, {
                timeOut: 2000,
                closeButton: true,
              });
            });
        }
      },
      (reason) => {
        console.log(`Dismissed: ${reason}`);
      }
    );
  }
}
