import { Injectable } from "@angular/core";
import { HttpClient, HttpHeaders } from "@angular/common/http";
import { Observable } from "rxjs";

const BASE_URL = 'https://kapi.absys.ninja/hemat';

@Injectable({
  providedIn: "root",
})
export class DeviceService {
  constructor(private http: HttpClient) {}

  getDeviceData(id, category, status): Observable<any> {
    const endpoint = `/devices`;
    const params = new URLSearchParams({
      page: "1",
      limit: "100",
      building_id: id ? id : "0",
      category_id: category ? category : "0",
      status_id: status ? status : "0",
      room_id: "0"
    });
    // if (category) {
    //   params.append("category_id", category);
    // }
    // if (status) {
    //   params.append("status_id", status);
    // }
    const url = `${BASE_URL}${endpoint}?${params.toString()}`;
    const headers = new HttpHeaders({
      "Content-Type": "application/json",
      "x-api-key": "j2yaYvPSQcsEEmHh3NEobfiXyyXmmnHT",
    });
    return this.http.get<any>(url, { headers });
  }

  getDeviceDataById(id): Observable<any> {
    const endpoint = `/devices`;
    const params = new URLSearchParams({
      page: "1",
      limit: "100",
      building_id: id
    });
    const url = `${BASE_URL}${endpoint}?${params.toString()}`;
    const headers = new HttpHeaders({
      "Content-Type": "application/json",
      "x-api-key": "j2yaYvPSQcsEEmHh3NEobfiXyyXmmnHT",
    });
    return this.http.get<any>(url, { headers });
  }

  getDeviceLIst(): Observable<any> {
    const endpoint = `/devices`;
    const url = `${BASE_URL}${endpoint}/list`;
    const headers = new HttpHeaders({
      "Content-Type": "application/json",
      "x-api-key": "j2yaYvPSQcsEEmHh3NEobfiXyyXmmnHT",
    });
    return this.http.get<any>(url, { headers });
  }

  getSyncDeviceData(): Observable<any> {
    const endpoint = `/devices`;
    const url = `${BASE_URL}${endpoint}/Synchronization`;
    const headers = new HttpHeaders({
      'Content-Type': 'application/json',
      'x-api-key': 'j2yaYvPSQcsEEmHh3NEobfiXyyXmmnHT'
    });
    return this.http.get<any>(url, { headers });
  }

  deviceSwitch(data): Observable<any> {
    const endpoint = `/devices`;
    const url = `${BASE_URL}${endpoint}/device-command`;
    const headers = new HttpHeaders({
      "Content-Type": "application/json",
      "x-api-key": "j2yaYvPSQcsEEmHh3NEobfiXyyXmmnHT",
    });
    return this.http.post<any>(url, data, { headers });
  }

  getStreamUrl(id): Observable<any> {
    const endpoint = `/devices`;
    const url = `${BASE_URL}${endpoint}/live-stream?device_id=${id}`;
    const headers = new HttpHeaders({
      'Content-Type': 'application/json',
      'x-api-key': 'j2yaYvPSQcsEEmHh3NEobfiXyyXmmnHT'
    });
    return this.http.get<any>(url, { headers });
  }

  getDeviceStatus(id): Observable<any> {
    const endpoint = `/devices`;
    const url = `${BASE_URL}${endpoint}/status?id=${id}`;
    const headers = new HttpHeaders({
      'Content-Type': 'application/json',
      'x-api-key': 'j2yaYvPSQcsEEmHh3NEobfiXyyXmmnHT'
    });
    return this.http.get<any>(url, { headers });
  }

  getDeviceByid(id): Observable<any> {
    const endpoint = `/devices`;
    const url = `${BASE_URL}${endpoint}/${id}`;
    const headers = new HttpHeaders({
      'Content-Type': 'application/json',
      'x-api-key': 'j2yaYvPSQcsEEmHh3NEobfiXyyXmmnHT'
    });
    return this.http.get<any>(url, { headers });
  }

  filterDashboard(building, floor): Observable<any> {
    const endpoint = `/building/dashboard/list`;
    const params = new URLSearchParams({
      building_id: building ? building : 0,
      floor_id: floor ? floor :0,
    });
    // if (category) {
    //   params.append("category_id", category);
    // }
    const url = `${BASE_URL}${endpoint}?${params.toString()}`;
    const headers = new HttpHeaders({
      "Content-Type": "application/json",
      "x-api-key": "j2yaYvPSQcsEEmHh3NEobfiXyyXmmnHT",
    });
    return this.http.get<any>(url, { headers });
  }

  getDeviceScheduler(id): Observable<any> {
    const endpoint = `/device-scheduler`;
    const url = `${BASE_URL}${endpoint}?device_id=${id}`;
    const headers = new HttpHeaders({
      'Content-Type': 'application/json',
      'x-api-key': 'j2yaYvPSQcsEEmHh3NEobfiXyyXmmnHT'
    });
    return this.http.get<any>(url, { headers });
  }

  postDeviceScheduler(data): Observable<any> {
    const endpoint = `/device-scheduler`;
    const url = `${BASE_URL}${endpoint}`;
    const headers = new HttpHeaders({
      "Content-Type": "application/json",
      "x-api-key": "j2yaYvPSQcsEEmHh3NEobfiXyyXmmnHT",
    });
    return this.http.post<any>(url, data, { headers });
  }

  putDeviceScheduler(data, id): Observable<any> {
    const endpoint = `/device-scheduler/${id}`;
    const url = `${BASE_URL}${endpoint}`;
    const headers = new HttpHeaders({
      "Content-Type": "application/json",
      "x-api-key": "j2yaYvPSQcsEEmHh3NEobfiXyyXmmnHT",
    });
    return this.http.put<any>(url, data, { headers });
  }

  putDeviceSchedulerActive(data, id): Observable<any> {
    const endpoint = `/device-scheduler/${id}/Active`;
    const url = `${BASE_URL}${endpoint}`;
    const headers = new HttpHeaders({
      "Content-Type": "application/json",
      "x-api-key": "j2yaYvPSQcsEEmHh3NEobfiXyyXmmnHT",
    });
    return this.http.put<any>(url, data, { headers });
  }

  deleteDeviceScheduler(id): Observable<any> {
    const endpoint = `/device-scheduler/${id}`;
    const url = `${BASE_URL}${endpoint}`;
    const headers = new HttpHeaders({
      "Content-Type": "application/json",
      "x-api-key": "j2yaYvPSQcsEEmHh3NEobfiXyyXmmnHT",
    });
    return this.http.delete<any>(url, { headers });
  }
}
