import { Injectable, NgZone } from '@angular/core';
import { Observable, Subject } from 'rxjs';

@Injectable({
  providedIn: 'root'
})
export class UserActivityService {
  private activitySubject = new Subject<boolean>();
  private timeoutId: any;

  constructor(private ngZone: NgZone) {
    this.setupActivityListeners();
  }

  private setupActivityListeners(): void {
    const events = ['click', 'mousemove', 'keypress'];
    events.forEach(event => {
      window.addEventListener(event, () => this.resetTimeout(), true);
    });
  }

  private resetTimeout(): void {
    this.ngZone.runOutsideAngular(() => {
      clearTimeout(this.timeoutId);
      this.timeoutId = setTimeout(() => this.activitySubject.next(false), 300000); // 5 minutes
      this.activitySubject.next(true);
    });
  }

  get activity$(): Observable<boolean> {
    return this.activitySubject.asObservable();
  }
}
